@@ -74,6 +74,10 @@ struct _AgsSoundProviderInterface
   void (*set_tempo)(AgsSoundProvider *sound_provider,
 		    GList *tempo);
   GList* (*get_tempo)(AgsSoundProvider *sound_provider);
+
+  void (*set_time_signature)(AgsSoundProvider *sound_provider,
+			     GList *time_signature);
+  GList* (*get_time_signature)(AgsSoundProvider *sound_provider);
 };
 
 GType ags_sound_provider_get_type();
@@ -114,6 +118,10 @@ void ags_sound_provider_set_tempo(AgsSoundProvider *sound_provider,
 				  GList *tempo);
 GList* ags_sound_provider_get_tempo(AgsSoundProvider *sound_provider);
 
+void ags_sound_provider_set_time_signature(AgsSoundProvider *sound_provider,
+					   GList *time_signature);
+GList* ags_sound_provider_get_time_signature(AgsSoundProvider *sound_provider);
+
 G_END_DECLS
 
 #endif /*__AGS_SOUND_PROVIDER_H__*/
